# Generated by Django 4.0.8 on 2022-12-22 06:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0003_alter_basetreenode_polymorphic_ctype_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='automate',
            name='dst_folder',
        ),
        migrations.RemoveField(
            model_name='automate',
            name='tags',
        ),
        migrations.RemoveField(
            model_name='automate',
            name='user',
        ),
        migrations.RemoveField(
            model_name='kvstorecompitem',
            name='comp_item',
        ),
#        migrations.RemoveField(
#            model_name='kvstorecompitem',
#            name='kvstore_ptr',
#        ),
        migrations.RemoveField(
            model_name='kvstorecompnode',
            name='node',
        ),
        migrations.RemoveField(
            model_name='kvstorenode',
            name='kvstore_ptr',
        ),
        migrations.RemoveField(
            model_name='kvstorenode',
            name='node',
        ),
        migrations.RemoveField(
            model_name='kvstorepage',
            name='kvstore_ptr',
        ),
        migrations.RemoveField(
            model_name='kvstorepage',
            name='page',
        ),
        migrations.DeleteModel(
            name='Access',
        ),
        migrations.DeleteModel(
            name='Automate',
        ),
        migrations.DeleteModel(
            name='KVStore',
        ),
        migrations.DeleteModel(
            name='KVStoreCompItem',
        ),
        migrations.DeleteModel(
            name='KVStoreCompNode',
        ),
        migrations.DeleteModel(
            name='KVStoreNode',
        ),
        migrations.DeleteModel(
            name='KVStorePage',
        ),
    ]
