# Generated by Django 4.0.8 on 2022-12-22 06:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0004_remove_automate_dst_folder_remove_automate_tags_and_more'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='basetreenode',
            constraint=models.UniqueConstraint(fields=('parent', 'title'), name='unique title per parent'),
        ),
    ]
