# Generated by Django 4.0.8 on 2022-12-30 07:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0005_basetreenode_unique title per parent'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='basetreenode',
            options={'verbose_name': 'Documents', 'verbose_name_plural': 'Documents'},
        ),
        migrations.RemoveField(
            model_name='basetreenode',
            name='level',
        ),
        migrations.RemoveField(
            model_name='basetreenode',
            name='lft',
        ),
        migrations.RemoveField(
            model_name='basetreenode',
            name='polymorphic_ctype',
        ),
        migrations.RemoveField(
            model_name='basetreenode',
            name='rght',
        ),
        migrations.RemoveField(
            model_name='basetreenode',
            name='tree_id',
        ),
        migrations.AddField(
            model_name='basetreenode',
            name='ctype',
            field=models.CharField(blank=True, choices=[('folder', 'folder'), ('document', 'document')], max_length=16, null=True),
        ),
    ]
